//+------------------------------------------------------------------+
//|                                                          M01.mq5 |
//|                        Copyright 2010, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2010, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
#property version   "1.00"
//+------------------------------------------------------------------+
//| Include                                                          |
//+------------------------------------------------------------------+
#include <Expert\Expert.mqh>
//--- available signals
#include <Expert\Signal\SignalAC.mqh>
//--- available trailing
#include <Expert\Trailing\TrailingFixedPips.mqh>
//--- available money management
#include <Expert\Money\MoneyNone.mqh>
//#include <MoyExpert.mqh>

//+------------------------------------------------------------------+
//| Inputs                                                           |
//+------------------------------------------------------------------+
//--- inputs for expert
input string Expert_Title                  ="Master02_ML"; // Document name

enum Cimv  //   
   {
    EURUSD,
    GBPUSD,
    AUDUSD,
    NZDUSD,
    USDCAD,
    USDCHF,
    USDJPY,
    AUDCAD,
    AUDJPY,
    GBPJPY,
    EURJPY,
    AUDNZD,
    AUDCHF,
    CHFJPY,
    EURCAD,
    EURGBP,
    GBPCHF,
    CADCHF,
    EURAUD,
    EURNZD,
    EURCHF,
    End
   };

input Cimv Symv1;      //  1
input Cimv Symv2;      //  2
input Cimv Symv3;      //  3

string Sim(Cimv Sim)  
 {                         
  if(Sim==EURUSD) return("EURUSD");
  if(Sim==GBPUSD) return("GBPUSD");
  if(Sim==AUDUSD) return("AUDUSD");
  if(Sim==NZDUSD) return("NZDUSD");
  if(Sim==USDCAD) return("USDCAD");
  if(Sim==USDCHF) return("USDCHF");
  if(Sim==USDJPY) return("USDJPY");
  if(Sim==AUDCAD) return("AUDCAD");
  if(Sim==AUDJPY) return("AUDJPY");
  if(Sim==GBPJPY) return("GBPJPY");
  if(Sim==EURJPY) return("EURJPY");
  if(Sim==AUDNZD) return("AUDNZD");
  if(Sim==AUDCHF) return("AUDCHF");
  if(Sim==CHFJPY) return("CHFJPY");
  if(Sim==EURCAD) return("EURCAD");
  if(Sim==EURGBP) return("EURGBP");
  if(Sim==GBPCHF) return("GBPCHF");
  if(Sim==CADCHF) return("CADCHF");
  if(Sim==EURAUD) return("EURAUD");
  if(Sim==EURNZD) return("EURNZD");
  if(Sim==EURCHF) return("EURCHF");
  return("");
 };

ulong        Expert_MagicNumber            =9557;  // 
bool         Expert_EveryTick              =false; // 
//--- inputs for main signal
input int    Signal_ThresholdOpen          =3;    // Signal threshold value to open [0...100]
input int    Signal_ThresholdClose         =21;    // Signal threshold value to close [0...100]
      double Signal_PriceLevel             =0.0;   // Price level to execute a deal
input double Signal_StopLevel              =100.0;  // Stop Loss level (in points)
      double Signal_TakeLevel              =0.0;  // Take Profit level (in points)
      int    Signal_Expiration             =0;     // Expiration of pending orders (in bars)
input double Signal_0_AC_Weight            =0.3;   // Accelerator Oscillator W1 Weight [0...1.0]
input double Signal_1_AC_Weight            =0.3;   // Accelerator Oscillator D1 Weight [0...1.0]
input double Signal_2_AC_Weight            =0.3;   // Accelerator Oscillator H4 Weight [0...1.0]
//--- inputs for trailing
input int    Trailing_FixedPips_StopLevel=100;   // Stop Loss trailing level (in points)
//+------------------------------------------------------------------+
//| Global expert object                                             |
//+------------------------------------------------------------------+

class AExpert : public CExpert        //     ,
  {                                    //     

public:
    bool  Init(string symbol,ENUM_TIMEFRAMES period,bool every_tick,ulong magic=0);
       
  };
//--------------------------------------------------------------------------------
bool AExpert::Init(string symbol,ENUM_TIMEFRAMES period,bool every_tick,ulong magic)
  {
//--- returns false if the EA is initialized on a symbol/timeframe different from the current one
 //  if(symbol!=Symbol() || period!=Period())
   if(period!=Period())
   
     {
      printf(__FUNCTION__+": wrong symbol or timeframe (must be %s:%s)",symbol,EnumToString(period));
      return(false);
     }
//--- initialize common information
   if(m_symbol==NULL)
     {
      if((m_symbol=new CSymbolInfo)==NULL) return(false);
     }
   if(!m_symbol.Name(symbol)) return(false);
   m_period    =period;                     // period
   m_every_tick=every_tick;
   if(every_tick)
      TimeframeAdd(WRONG_VALUE);            // add all periods
   else
      TimeframeAdd(period);                 // add specified period
//--- tuning for 3 or 5 digits
   int digits_adjust=1;
   if(m_symbol.Digits()==3 || m_symbol.Digits()==5) digits_adjust=10;
   m_adjusted_point=m_symbol.Point()*digits_adjust;
//--- initializing objects expert
   if(!InitTrade(magic))
     {
      printf(__FUNCTION__+": error initialization trade object");
      return(false);
     }
   if(!InitSignal())
     {
      printf(__FUNCTION__+": error initialization signal object");
      return(false);
     }
   if(!InitTrailing())
     {
      printf(__FUNCTION__+": error initialization trailing object");
      return(false);
     }
   if(!InitMoney())
     {
      printf(__FUNCTION__+": error initialization money object");
      return(false);
     }
   if(!InitParameters())
     {
      printf(__FUNCTION__+": error initialization parameters");
      return(false);
     }
//--- initialization for working with trade history
   PrepareHistoryDate();
   HistoryPoint();
//--- primary initialization is successful, pass to the phase of tuning
   m_init_phase    =INIT_PHASE_TUNING;
//--- ok
   return(true);
  }
  
//-----------------------------------------------------------------------
AExpert ExtExpert_EUs;
AExpert ExtExpert_AJp;
AExpert ExtExpert_EJp;
//+------------------------------------------------------------------+
//| Initialization function of the expert                            |
//+------------------------------------------------------------------+
int OnInit()
  {
//================================================================EURUSD
 if(Symv1!=End)
 { 
//--- Initializing expert
   if(!ExtExpert_EUs.Init(Sim(Symv1),Period(),Expert_EveryTick,Expert_MagicNumber))
     {
      //--- failed
      printf(__FUNCTION__+": error initializing expert_EUs");
      ExtExpert_EUs.Deinit();
      return(-1);
     }

//--- Creating signal
   CExpertSignal *signal_EUs=new CExpertSignal;
   if(signal_EUs==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating signal_EUs");
      ExtExpert_EUs.Deinit();
      return(-2);
     }
     
//---
   ExtExpert_EUs.InitSignal(signal_EUs);
   signal_EUs.ThresholdOpen(Signal_ThresholdOpen);
   signal_EUs.ThresholdClose(Signal_ThresholdClose);
   signal_EUs.PriceLevel(Signal_PriceLevel);
   signal_EUs.StopLevel(Signal_StopLevel);
   signal_EUs.TakeLevel(Signal_TakeLevel);
   signal_EUs.Expiration(Signal_Expiration);
//--- Creating filter CSignalAC
   CSignalAC *filter_EUs0=new CSignalAC;
   if(filter_EUs0==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating filter_EUs0");
      ExtExpert_EUs.Deinit();
      return(-3);
     }
     
   signal_EUs.AddFilter(filter_EUs0);
//--- Set filter parameters
   filter_EUs0.Period(PERIOD_W1);
   filter_EUs0.Weight(Signal_0_AC_Weight);
//--- Creating filter CSignalAC
   CSignalAC *filter_EUs1=new CSignalAC;
   if(filter_EUs1==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating filter_EUs1");
      ExtExpert_EUs.Deinit();
      return(-4);
     }
     
   signal_EUs.AddFilter(filter_EUs1);
//--- Set filter parameters
   filter_EUs1.Period(PERIOD_D1);
   filter_EUs1.Weight(Signal_1_AC_Weight);
//--- Creating filter CSignalAC
   CSignalAC *filter_EUs2=new CSignalAC;
   if(filter_EUs2==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating filter_EUs2");
      ExtExpert_EUs.Deinit();
      return(-5);
     }
     
   signal_EUs.AddFilter(filter_EUs2);
//--- Set filter parameters
   filter_EUs2.Period(PERIOD_H4);
   filter_EUs2.Weight(Signal_2_AC_Weight);
//--- Creation of trailing object
   CTrailingFixedPips *trailing_EUs=new CTrailingFixedPips;
   if(trailing_EUs==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating trailing_EUs");
      ExtExpert_EUs.Deinit();
      return(-6);
     }
     
//--- Add trailing to expert (will be deleted automatically))
   if(!ExtExpert_EUs.InitTrailing(trailing_EUs))
     {
      //--- failed
      printf(__FUNCTION__+": error initializing trailing_EUs");
      ExtExpert_EUs.Deinit();
      return(-7);
     }
     
//--- Set trailing parameters
   trailing_EUs.StopLevel(Trailing_FixedPips_StopLevel);
   trailing_EUs.ProfitLevel(0);
//--- Creation of money object
   CMoneyNone *money_EUs=new CMoneyNone;
   if(money_EUs==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating money_EUs");
      ExtExpert_EUs.Deinit();
      return(-8);
     }
     
//--- Add money to expert (will be deleted automatically))
   if(!ExtExpert_EUs.InitMoney(money_EUs))
     {
      //--- failed
      printf(__FUNCTION__+": error initializing money_EUs");
      ExtExpert_EUs.Deinit();
      return(-9);
     }
     
//--- Set money parameters
//--- Check all trading objects parameters
   if(!ExtExpert_EUs.ValidationSettings())
     {
      //--- failed
      ExtExpert_EUs.Deinit();
      return(-10);
     }
     
//--- Tuning of all necessary indicators
   if(!ExtExpert_EUs.InitIndicators())
     {
      //--- failed
      printf(__FUNCTION__+": error initializing indicators");
      ExtExpert_EUs.Deinit();
      return(-11);
     }
} // Symv1     
//=======================================================================AUPDJPY
 if(Symv2!=End)
 { 
//--- Initializing expert
   if(!ExtExpert_AJp.Init(Sim(Symv2),Period(),Expert_EveryTick,Expert_MagicNumber))
     {
      //--- failed
      printf(__FUNCTION__+": error initializing expert_AJp");
      ExtExpert_AJp.Deinit();
      return(-1);
     }
     
//--- Creating signal
   CExpertSignal *signal_AJp=new CExpertSignal;
   if(signal_AJp==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating signal_AJp");
      ExtExpert_AJp.Deinit();
      return(-2);
     }
//---
   ExtExpert_AJp.InitSignal(signal_AJp);
   signal_AJp.ThresholdOpen(Signal_ThresholdOpen);
   signal_AJp.ThresholdClose(Signal_ThresholdClose);
   signal_AJp.PriceLevel(Signal_PriceLevel);
   signal_AJp.StopLevel(Signal_StopLevel);
   signal_AJp.TakeLevel(Signal_TakeLevel);
   signal_AJp.Expiration(Signal_Expiration);
//--- Creating filter CSignalAC
   CSignalAC *filter_AJp0=new CSignalAC;
   if(filter_AJp0==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating filter_AJp0");
      ExtExpert_AJp.Deinit();
      return(-3);
     }
   signal_AJp.AddFilter(filter_AJp0);
//--- Set filter parameters
   filter_AJp0.Period(PERIOD_W1);
   filter_AJp0.Weight(Signal_0_AC_Weight);
//--- Creating filter CSignalAC
   CSignalAC *filter_AJp1=new CSignalAC;
   if(filter_AJp1==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating filter_AJp1");
      ExtExpert_AJp.Deinit();
      return(-4);
     }
   signal_AJp.AddFilter(filter_AJp1);
//--- Set filter parameters
   filter_AJp1.Period(PERIOD_D1);
   filter_AJp1.Weight(Signal_1_AC_Weight);
//--- Creating filter CSignalAC
   CSignalAC *filter_AJp2=new CSignalAC;
   if(filter_AJp2==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating filter_AJp2");
      ExtExpert_AJp.Deinit();
      return(-5);
     }
   signal_AJp.AddFilter(filter_AJp2);
//--- Set filter parameters
   filter_AJp2.Period(PERIOD_H4);
   filter_AJp2.Weight(Signal_2_AC_Weight);
//--- Creation of trailing object
   CTrailingFixedPips *trailing_AJp=new CTrailingFixedPips;
   if(trailing_AJp==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating trailing_AJp");
      ExtExpert_AJp.Deinit();
      return(-6);
     }
//--- Add trailing to expert (will be deleted automatically))
   if(!ExtExpert_AJp.InitTrailing(trailing_AJp))
     {
      //--- failed
      printf(__FUNCTION__+": error initializing trailing_AJp");
      ExtExpert_AJp.Deinit();
      return(-7);
     }
//--- Set trailing parameters
   trailing_AJp.StopLevel(Trailing_FixedPips_StopLevel);
   trailing_AJp.ProfitLevel(0);
//--- Creation of money object
   CMoneyNone *money_AJp=new CMoneyNone;
   if(money_AJp==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating money_AJp");
      ExtExpert_AJp.Deinit();
      return(-8);
     }
//--- Add money to expert (will be deleted automatically))
   if(!ExtExpert_AJp.InitMoney(money_AJp))
     {
      //--- failed
      printf(__FUNCTION__+": error initializing money_AJp");
      ExtExpert_AJp.Deinit();
      return(-9);
     }
//--- Set money parameters
//--- Check all trading objects parameters
   if(!ExtExpert_AJp.ValidationSettings())
     {
      //--- failed
      ExtExpert_AJp.Deinit();
      return(-10);
     }
//--- Tuning of all necessary indicators
   if(!ExtExpert_AJp.InitIndicators())
     {
      //--- failed
      printf(__FUNCTION__+": error initializing indicators");
      ExtExpert_AJp.Deinit();
      return(-11);
     }
 } // Simv2    
//==============================================================================EURJPY
 if(Symv3!=End)
 { 
//--- Initializing expert
   if(!ExtExpert_EJp.Init(Sim(Symv3),Period(),Expert_EveryTick,Expert_MagicNumber))
     {
      //--- failed
      printf(__FUNCTION__+": error initializing expert_EJp");
      ExtExpert_EJp.Deinit();
      return(-1);
     }
     
//--- Creating signal
   CExpertSignal *signal_EJp=new CExpertSignal;
   if(signal_EJp==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating signal_EJp");
      ExtExpert_EJp.Deinit();
      return(-2);
     }
//---
   ExtExpert_EJp.InitSignal(signal_EJp);
   signal_EJp.ThresholdOpen(Signal_ThresholdOpen);
   signal_EJp.ThresholdClose(Signal_ThresholdClose);
   signal_EJp.PriceLevel(Signal_PriceLevel);
   signal_EJp.StopLevel(Signal_StopLevel);
   signal_EJp.TakeLevel(Signal_TakeLevel);
   signal_EJp.Expiration(Signal_Expiration);
//--- Creating filter CSignalAC
   CSignalAC *filter_EJp0=new CSignalAC;
   if(filter_EJp0==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating filter_EJp0");
      ExtExpert_EJp.Deinit();
      return(-3);
     }
   signal_EJp.AddFilter(filter_EJp0);
//--- Set filter parameters
   filter_EJp0.Period(PERIOD_W1);
   filter_EJp0.Weight(Signal_0_AC_Weight);
//--- Creating filter CSignalAC
   CSignalAC *filter_EJp1=new CSignalAC;
   if(filter_EJp1==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating filter_EJp1");
      ExtExpert_EJp.Deinit();
      return(-4);
     }
   signal_EJp.AddFilter(filter_EJp1);
//--- Set filter parameters
   filter_EJp1.Period(PERIOD_D1);
   filter_EJp1.Weight(Signal_1_AC_Weight);
//--- Creating filter CSignalAC
   CSignalAC *filter_EJp2=new CSignalAC;
   if(filter_EJp2==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating filter_EJp2");
      ExtExpert_EJp.Deinit();
      return(-5);
     }
   signal_EJp.AddFilter(filter_EJp2);
//--- Set filter parameters
   filter_EJp2.Period(PERIOD_H4);
   filter_EJp2.Weight(Signal_2_AC_Weight);
//--- Creation of trailing object
   CTrailingFixedPips *trailing_EJp=new CTrailingFixedPips;
   if(trailing_EJp==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating trailing_EJp");
      ExtExpert_EJp.Deinit();
      return(-6);
     }
//--- Add trailing to expert (will be deleted automatically))
   if(!ExtExpert_EJp.InitTrailing(trailing_EJp))
     {
      //--- failed
      printf(__FUNCTION__+": error initializing trailing_EJp");
      ExtExpert_EJp.Deinit();
      return(-7);
     }
//--- Set trailing parameters
   trailing_EJp.StopLevel(Trailing_FixedPips_StopLevel);
   trailing_EJp.ProfitLevel(0);
//--- Creation of money object
   CMoneyNone *money_EJp=new CMoneyNone;
   if(money_EJp==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating money_EJp");
      ExtExpert_EJp.Deinit();
      return(-8);
     }
//--- Add money to expert (will be deleted automatically))
   if(!ExtExpert_EJp.InitMoney(money_EJp))
     {
      //--- failed
      printf(__FUNCTION__+": error initializing money_EJp");
      ExtExpert_EJp.Deinit();
      return(-9);
     }
//--- Set money parameters
//--- Check all trading objects parameters
   if(!ExtExpert_EJp.ValidationSettings())
     {
      //--- failed
      ExtExpert_EJp.Deinit();
      return(-10);
     }
//--- Tuning of all necessary indicators
   if(!ExtExpert_EJp.InitIndicators())
     {
      //--- failed
      printf(__FUNCTION__+": error initializing indicators");
      ExtExpert_EJp.Deinit();
      return(-11);
     }
} // Symv3     
//====================================================================================     
//--- ok
   return(0);
  }
//+------------------------------------------------------------------+
//| Deinitialization function of the expert                          |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
   if(Symv1!=End) ExtExpert_EUs.Deinit();
   if(Symv2!=End) ExtExpert_AJp.Deinit();
   if(Symv3!=End) ExtExpert_EJp.Deinit();
  }
//+------------------------------------------------------------------+
//| "Tick" event handler function                                    |
//+------------------------------------------------------------------+
void OnTick()
  {
   if(Symv1!=End) ExtExpert_EUs.OnTick();
   if(Symv2!=End) ExtExpert_AJp.OnTick();
   if(Symv3!=End) ExtExpert_EJp.OnTick();
  }
//+------------------------------------------------------------------+
//| "Trade" event handler function                                   |
//+------------------------------------------------------------------+
void OnTrade()
  {
   if(Symv1!=End) ExtExpert_EUs.OnTrade();
   if(Symv2!=End) ExtExpert_AJp.OnTrade();
   if(Symv3!=End) ExtExpert_EJp.OnTrade();
  }
//+------------------------------------------------------------------+
//| "Timer" event handler function                                   |
//+------------------------------------------------------------------+
void OnTimer()
  {
   if(Symv1!=End) ExtExpert_EUs.OnTimer();
   if(Symv2!=End) ExtExpert_AJp.OnTimer();
   if(Symv3!=End) ExtExpert_EJp.OnTimer();
  }
//+------------------------------------------------------------------+
